/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.database;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Query;

public class BatchDatabaseManager implements DatabaseManager {
  // IMPORTANT : this value must be the same than the property hibernate.jdbc.batch_size from /META-INF/persistence.xml :
  public static final int BATCH_SIZE = 100;

  private DatabaseConnector connector;
  private EntityManager entityManager = null;
  private int index = 0;
  private boolean inTransaction = false;

  public BatchDatabaseManager(DatabaseConnector connector) {
    this.connector = connector;
  }

  public EntityManager getEntityManager() {
    return entityManager;
  }


  public void start() {
    entityManager = connector.createEntityManager();
    index = 0;
  }

  public void stop() {
    commit();
    if (entityManager != null && entityManager.isOpen()) {
      entityManager.clear();
      entityManager.close();
      entityManager = null;
    }
  }

  public void commit() {
    if (entityManager != null && inTransaction) {
      flush();
      if (entityManager.getTransaction().getRollbackOnly()) {
        entityManager.getTransaction().rollback();
      } else {
        entityManager.getTransaction().commit();
      }
      inTransaction = false;
      index = 0;
    }
  }

  public void rollback() {
    if (entityManager != null && inTransaction) {
      entityManager.getTransaction().rollback();
      inTransaction = false;
      index = 0;
    }
  }


  public Object save(Object model) {
    startTransaction();
    entityManager.persist(model);
    if (++index % BATCH_SIZE == 0) {
      flush();
    }
    return model;
  }

  public Object merge(Object model) {
    startTransaction();
    return entityManager.merge(model);
  }

  public void remove(Object model) {
    startTransaction();
    entityManager.remove(model);
    if (++index % BATCH_SIZE == 0) {
      flush();
    }
  }

  public <T> T reattach(Class<T> entityClass, Object primaryKey) {
    startTransaction();
    return entityManager.getReference(entityClass, primaryKey);
  }

  private void startTransaction() {
    if (!inTransaction) {
      entityManager.getTransaction().begin();
      inTransaction = true;
    }
  }

  private void flush() {
    entityManager.flush();
    entityManager.clear();
  }

  public Query createQuery(String hql) {
    startTransaction();
    return entityManager.createQuery(hql);
  }

  public Query createNamedQuery(String hql) {
    startTransaction();
    return entityManager.createNamedQuery(hql);
  }

  public Object getSingleResult(Query query, Object defaultValue) {
    try {
      return query.getSingleResult();
    }
    catch (NoResultException ex) {
      return defaultValue;
    }
  }
}
